@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,17 +13,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import abc
-import typing
+from typing import Awaitable, Callable, Dict, Optional, Sequence, Union
+import packaging.version
 import pkg_resources
 
-from google import auth  # type: ignore
-from google.api_core import exceptions  # type: ignore
+import google.auth  # type: ignore
+import google.api_core  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
 from google.api_core import operations_v1  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 
 from google.cloud.artifactregistry_v1beta2.types import file
 from google.cloud.artifactregistry_v1beta2.types import package
@@ -33,11 +33,10 @@ from google.cloud.artifactregistry_v1beta2.types import repository as gda_reposi
 from google.cloud.artifactregistry_v1beta2.types import tag
 from google.cloud.artifactregistry_v1beta2.types import tag as gda_tag
 from google.cloud.artifactregistry_v1beta2.types import version
-from google.iam.v1 import iam_policy_pb2 as iam_policy  # type: ignore
-from google.iam.v1 import policy_pb2 as policy  # type: ignore
-from google.longrunning import operations_pb2 as operations  # type: ignore
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.iam.v1 import iam_policy_pb2  # type: ignore
+from google.iam.v1 import policy_pb2  # type: ignore
+from google.longrunning import operations_pb2  # type: ignore
+from google.protobuf import empty_pb2  # type: ignore
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
@@ -48,6 +47,17 @@ try:
 except pkg_resources.DistributionNotFound:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()
 
+try:
+    # google.auth.__version__ was added in 1.26.0
+    _GOOGLE_AUTH_VERSION = google.auth.__version__
+except AttributeError:
+    try:  # try pkg_resources if it is available
+        _GOOGLE_AUTH_VERSION = pkg_resources.get_distribution("google-auth").version
+    except pkg_resources.DistributionNotFound:  # pragma: NO COVER
+        _GOOGLE_AUTH_VERSION = None
+
+_API_CORE_VERSION = google.api_core.__version__
+
 
 class ArtifactRegistryTransport(abc.ABC):
     """Abstract transport class for ArtifactRegistry."""
@@ -57,21 +67,24 @@ class ArtifactRegistryTransport(abc.ABC):
         "https://www.googleapis.com/auth/cloud-platform.read-only",
     )
 
+    DEFAULT_HOST: str = "artifactregistry.googleapis.com"
+
     def __init__(
         self,
         *,
-        host: str = "artifactregistry.googleapis.com",
-        credentials: credentials.Credentials = None,
-        credentials_file: typing.Optional[str] = None,
-        scopes: typing.Optional[typing.Sequence[str]] = AUTH_SCOPES,
-        quota_project_id: typing.Optional[str] = None,
+        host: str = DEFAULT_HOST,
+        credentials: ga_credentials.Credentials = None,
+        credentials_file: Optional[str] = None,
+        scopes: Optional[Sequence[str]] = None,
+        quota_project_id: Optional[str] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
         **kwargs,
     ) -> None:
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -80,13 +93,13 @@ class ArtifactRegistryTransport(abc.ABC):
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
                 This argument is mutually exclusive with credentials.
-            scope (Optional[Sequence[str]]): A list of scopes.
+            scopes (Optional[Sequence[str]]): A list of scopes.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
-            client_info (google.api_core.gapic_v1.client_info.ClientInfo):	
-                The client info used to send a user-agent string along with	
-                API requests. If ``None``, then default info will be used.	
-                Generally, you only need to set this if you're developing	
+            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
+                The client info used to send a user-agent string along with
+                API requests. If ``None``, then default info will be used.
+                Generally, you only need to set this if you're developing
                 your own client library.
         """
         # Save the hostname. Default to port 443 (HTTPS) if none is specified.
@@ -94,28 +107,75 @@ class ArtifactRegistryTransport(abc.ABC):
             host += ":443"
         self._host = host
 
+        scopes_kwargs = self._get_scopes_kwargs(self._host, scopes)
+
+        # Save the scopes.
+        self._scopes = scopes or self.AUTH_SCOPES
+
         # If no credentials are provided, then determine the appropriate
         # defaults.
         if credentials and credentials_file:
-            raise exceptions.DuplicateCredentialArgs(
+            raise core_exceptions.DuplicateCredentialArgs(
                 "'credentials_file' and 'credentials' are mutually exclusive"
             )
 
         if credentials_file is not None:
-            credentials, _ = auth.load_credentials_from_file(
-                credentials_file, scopes=scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.load_credentials_from_file(
+                credentials_file, **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         elif credentials is None:
-            credentials, _ = auth.default(
-                scopes=scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.default(
+                **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         # Save the credentials.
         self._credentials = credentials
 
-        # Lifted into its own function so it can be stubbed out during tests.
-        self._prep_wrapped_messages(client_info)
+    # TODO(busunkim): These two class methods are in the base transport
+    # to avoid duplicating code across the transport classes. These functions
+    # should be deleted once the minimum required versions of google-api-core
+    # and google-auth are increased.
+
+    # TODO: Remove this function once google-auth >= 1.25.0 is required
+    @classmethod
+    def _get_scopes_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Optional[Sequence[str]]]:
+        """Returns scopes kwargs to pass to google-auth methods depending on the google-auth version"""
+
+        scopes_kwargs = {}
+
+        if _GOOGLE_AUTH_VERSION and (
+            packaging.version.parse(_GOOGLE_AUTH_VERSION)
+            >= packaging.version.parse("1.25.0")
+        ):
+            scopes_kwargs = {"scopes": scopes, "default_scopes": cls.AUTH_SCOPES}
+        else:
+            scopes_kwargs = {"scopes": scopes or cls.AUTH_SCOPES}
+
+        return scopes_kwargs
+
+    # TODO: Remove this function once google-api-core >= 1.26.0 is required
+    @classmethod
+    def _get_self_signed_jwt_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Union[Optional[Sequence[str]], str]]:
+        """Returns kwargs to pass to grpc_helpers.create_channel depending on the google-api-core version"""
+
+        self_signed_jwt_kwargs: Dict[str, Union[Optional[Sequence[str]], str]] = {}
+
+        if _API_CORE_VERSION and (
+            packaging.version.parse(_API_CORE_VERSION)
+            >= packaging.version.parse("1.26.0")
+        ):
+            self_signed_jwt_kwargs["default_scopes"] = cls.AUTH_SCOPES
+            self_signed_jwt_kwargs["scopes"] = scopes
+            self_signed_jwt_kwargs["default_host"] = cls.DEFAULT_HOST
+        else:
+            self_signed_jwt_kwargs["scopes"] = scopes or cls.AUTH_SCOPES
+
+        return self_signed_jwt_kwargs
 
     def _prep_wrapped_messages(self, client_info):
         # Precompute the wrapped methods.
@@ -126,7 +186,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -137,7 +200,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -154,7 +220,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -165,7 +234,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -176,7 +248,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -187,7 +262,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -198,7 +276,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -209,7 +290,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -220,7 +304,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -231,7 +318,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -242,7 +332,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -253,7 +346,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -264,7 +360,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -281,7 +380,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -295,7 +397,10 @@ class ArtifactRegistryTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=30.0,
                 ),
                 default_timeout=30.0,
                 client_info=client_info,
@@ -315,11 +420,11 @@ class ArtifactRegistryTransport(abc.ABC):
     @property
     def list_repositories(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [repository.ListRepositoriesRequest],
-        typing.Union[
+        Union[
             repository.ListRepositoriesResponse,
-            typing.Awaitable[repository.ListRepositoriesResponse],
+            Awaitable[repository.ListRepositoriesResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -327,185 +432,171 @@ class ArtifactRegistryTransport(abc.ABC):
     @property
     def get_repository(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [repository.GetRepositoryRequest],
-        typing.Union[repository.Repository, typing.Awaitable[repository.Repository]],
+        Union[repository.Repository, Awaitable[repository.Repository]],
     ]:
         raise NotImplementedError()
 
     @property
     def create_repository(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [gda_repository.CreateRepositoryRequest],
-        typing.Union[operations.Operation, typing.Awaitable[operations.Operation]],
+        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def update_repository(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [gda_repository.UpdateRepositoryRequest],
-        typing.Union[
-            gda_repository.Repository, typing.Awaitable[gda_repository.Repository]
-        ],
+        Union[gda_repository.Repository, Awaitable[gda_repository.Repository]],
     ]:
         raise NotImplementedError()
 
     @property
     def delete_repository(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [repository.DeleteRepositoryRequest],
-        typing.Union[operations.Operation, typing.Awaitable[operations.Operation]],
+        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_packages(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [package.ListPackagesRequest],
-        typing.Union[
-            package.ListPackagesResponse, typing.Awaitable[package.ListPackagesResponse]
-        ],
+        Union[package.ListPackagesResponse, Awaitable[package.ListPackagesResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_package(
         self,
-    ) -> typing.Callable[
-        [package.GetPackageRequest],
-        typing.Union[package.Package, typing.Awaitable[package.Package]],
+    ) -> Callable[
+        [package.GetPackageRequest], Union[package.Package, Awaitable[package.Package]]
     ]:
         raise NotImplementedError()
 
     @property
     def delete_package(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [package.DeletePackageRequest],
-        typing.Union[operations.Operation, typing.Awaitable[operations.Operation]],
+        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_versions(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [version.ListVersionsRequest],
-        typing.Union[
-            version.ListVersionsResponse, typing.Awaitable[version.ListVersionsResponse]
-        ],
+        Union[version.ListVersionsResponse, Awaitable[version.ListVersionsResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_version(
         self,
-    ) -> typing.Callable[
-        [version.GetVersionRequest],
-        typing.Union[version.Version, typing.Awaitable[version.Version]],
+    ) -> Callable[
+        [version.GetVersionRequest], Union[version.Version, Awaitable[version.Version]]
     ]:
         raise NotImplementedError()
 
     @property
     def delete_version(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [version.DeleteVersionRequest],
-        typing.Union[operations.Operation, typing.Awaitable[operations.Operation]],
+        Union[operations_pb2.Operation, Awaitable[operations_pb2.Operation]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_files(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [file.ListFilesRequest],
-        typing.Union[file.ListFilesResponse, typing.Awaitable[file.ListFilesResponse]],
+        Union[file.ListFilesResponse, Awaitable[file.ListFilesResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_file(
         self,
-    ) -> typing.Callable[
-        [file.GetFileRequest], typing.Union[file.File, typing.Awaitable[file.File]]
-    ]:
+    ) -> Callable[[file.GetFileRequest], Union[file.File, Awaitable[file.File]]]:
         raise NotImplementedError()
 
     @property
     def list_tags(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [tag.ListTagsRequest],
-        typing.Union[tag.ListTagsResponse, typing.Awaitable[tag.ListTagsResponse]],
+        Union[tag.ListTagsResponse, Awaitable[tag.ListTagsResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_tag(
         self,
-    ) -> typing.Callable[
-        [tag.GetTagRequest], typing.Union[tag.Tag, typing.Awaitable[tag.Tag]]
-    ]:
+    ) -> Callable[[tag.GetTagRequest], Union[tag.Tag, Awaitable[tag.Tag]]]:
         raise NotImplementedError()
 
     @property
     def create_tag(
         self,
-    ) -> typing.Callable[
-        [gda_tag.CreateTagRequest],
-        typing.Union[gda_tag.Tag, typing.Awaitable[gda_tag.Tag]],
+    ) -> Callable[
+        [gda_tag.CreateTagRequest], Union[gda_tag.Tag, Awaitable[gda_tag.Tag]]
     ]:
         raise NotImplementedError()
 
     @property
     def update_tag(
         self,
-    ) -> typing.Callable[
-        [gda_tag.UpdateTagRequest],
-        typing.Union[gda_tag.Tag, typing.Awaitable[gda_tag.Tag]],
+    ) -> Callable[
+        [gda_tag.UpdateTagRequest], Union[gda_tag.Tag, Awaitable[gda_tag.Tag]]
     ]:
         raise NotImplementedError()
 
     @property
     def delete_tag(
         self,
-    ) -> typing.Callable[
-        [tag.DeleteTagRequest], typing.Union[empty.Empty, typing.Awaitable[empty.Empty]]
+    ) -> Callable[
+        [tag.DeleteTagRequest], Union[empty_pb2.Empty, Awaitable[empty_pb2.Empty]]
     ]:
         raise NotImplementedError()
 
     @property
     def set_iam_policy(
         self,
-    ) -> typing.Callable[
-        [iam_policy.SetIamPolicyRequest],
-        typing.Union[policy.Policy, typing.Awaitable[policy.Policy]],
+    ) -> Callable[
+        [iam_policy_pb2.SetIamPolicyRequest],
+        Union[policy_pb2.Policy, Awaitable[policy_pb2.Policy]],
     ]:
         raise NotImplementedError()
 
     @property
     def get_iam_policy(
         self,
-    ) -> typing.Callable[
-        [iam_policy.GetIamPolicyRequest],
-        typing.Union[policy.Policy, typing.Awaitable[policy.Policy]],
+    ) -> Callable[
+        [iam_policy_pb2.GetIamPolicyRequest],
+        Union[policy_pb2.Policy, Awaitable[policy_pb2.Policy]],
     ]:
         raise NotImplementedError()
 
     @property
     def test_iam_permissions(
         self,
-    ) -> typing.Callable[
-        [iam_policy.TestIamPermissionsRequest],
-        typing.Union[
-            iam_policy.TestIamPermissionsResponse,
-            typing.Awaitable[iam_policy.TestIamPermissionsResponse],
+    ) -> Callable[
+        [iam_policy_pb2.TestIamPermissionsRequest],
+        Union[
+            iam_policy_pb2.TestIamPermissionsResponse,
+            Awaitable[iam_policy_pb2.TestIamPermissionsResponse],
         ],
     ]:
         raise NotImplementedError()
